program Slowness

  use sep
  use dottest
  use cgstep_mod
  use slowness_1D
  use weight_mod
  use solver_smp_mod

  implicit none 
  integer                                 :: ndim, nt, nx, nh, nm, niter, stat
  real                                    :: t0,dt,x0,dx,h0,dh,dm,mf,m0,maxoffset
  real,    dimension(2)                   :: dot1,dot2
  real,    dimension (:)    , allocatable :: model   ! Estimated velocity perturbations
  real,    dimension (:)    , allocatable :: data    ! CMP gathers
  real,    dimension (:)    , allocatable :: weight  ! Weighting function
  real,    dimension (:)    , pointer     :: s0      ! Initial 1D velocity
  integer                                 :: j, verb
  integer, dimension (:)    , allocatable :: n
  real,    dimension (:)    , allocatable :: d, o
  real,    dimension (:,:)  , allocatable :: rmov, mmov ! Movies

  call initpar  ()                   ! start processing parameters
  call sep_init (SOURCE)
  ndim=sep_dimension()
  allocate(n(ndim),o(ndim),d(ndim))
  
  call from_history('n',n)
  call from_history('d',d)
  call from_history('o',o)
  call from_param('verb',verb,0)
  call from_param('niter',niter,5)
  
  nt=n(1);t0=o(1);dt=d(1) ! Time
  nh=n(2);h0=o(2);dh=d(2) ! Offset
  nx=n(3);x0=o(3);dx=d(3) ! CMP

!  if (dx.ne.dh) then
!     write(0,*) '< ================== >'
!     write(0,*) '< = ERROR DETECTED = >'
!     write(0,*) '< = CMP and offset should have the same step size, good bye = >'
!     write(0,*) '< ================== >'
!     call erexit(0)
!  end if
  
  maxoffset=h0+nh*dh       
  m0=x0-maxoffset/2       ! New origin of the model
  mf=x0+dx*nx+maxoffset/2 ! End of the model
  nm=ceiling((mf-m0)/dx)  ! Final size of the velocity model
  dm=dx                   ! Assumes CMP and velocity on the same grid
 
  if (verb.ge.1) write(0,*)
  if (verb.ge.1) write(0,*)
  if (verb.ge.1) write(0,*) '< ========================= >'
  if (verb.ge.1) write(0,*) '< = MODEL SPACE PARAMETERS= >'
  if (verb.ge.1) write(0,*) '< ========================= >'
  if (verb.ge.1) write(0,*) '< = o-time    = >',t0
  if (verb.ge.1) write(0,*) '< = d-time    = >',dt
  if (verb.ge.1) write(0,*) '< = n-time    = >',nt
  if (verb.ge.1) write(0,*) '< = o-cmp     = >',m0
  if (verb.ge.1) write(0,*) '< = n-cmp     = >',nm
  if (verb.ge.1) write(0,*) '< = d-cmp     = >',dm
  if (verb.ge.1) write(0,*) '< ========================= >'
  if (verb.ge.1) write(0,*)
  if (verb.ge.1) write(0,*)
  if (verb.ge.1) write(0,*) '< ========================= >'
  if (verb.ge.1) write(0,*) '< = DATA SPACE PARAMETERS = >'
  if (verb.ge.1) write(0,*) '< ========================= >'
  if (verb.ge.1) write(0,*) '< = o-time    = >',t0
  if (verb.ge.1) write(0,*) '< = d-time    = >',dt
  if (verb.ge.1) write(0,*) '< = n-time    = >',nt
  if (verb.ge.1) write(0,*) '< = o-offset  = >',h0
  if (verb.ge.1) write(0,*) '< = d-offset  = >',dh
  if (verb.ge.1) write(0,*) '< = n-offset  = >',nh
  if (verb.ge.1) write(0,*) '< = o-cmp     = >',x0
  if (verb.ge.1) write(0,*) '< = d-cmp     = >',dx
  if (verb.ge.1) write(0,*) '< = n-cmp     = >',nx
  if (verb.ge.1) write(0,*) '< ========================= >'
  if (verb.ge.1) write(0,*)
  if (verb.ge.1) write(0,*)
  if (verb.ge.1) write(0,*) '< ========================= >'
  if (verb.ge.1) write(0,*) '< = INVERSION  PARAMETERS = >'
  if (verb.ge.1) write(0,*) '< ========================= >'
  if (verb.ge.1) write(0,*) '< = niter    = >',niter
  if (verb.ge.1) write(0,*) '< ========================= >'
  if (verb.ge.1) write(0,*)
  if (verb.ge.1) write(0,*)
  
  allocate (data(nt*nh*nx), model(nt*nm), s0(nt),mmov(nt*nm,niter),rmov(nt*nh*nx,niter))
   
!  allocate (data(nt*nh*nx), model(nt*nx), s0(nt,nx),mmov(nt*nx,niter),rmov(nt*nh*nx,niter))
  
  
  if (exist_file("weight")) then
     allocate(weight(nt*nh*nx))
     call sep_read(weight,'weight')
  end if

  call sep_read(data)
  call sep_read(s0,'s0')

  call slowness_init(t0,dt,nt,x0,dx,nx,m0,dm,nm,h0,dh,nh,s0,verb)

  if (exist_file("weight")) then
     call weight_init(nt*nh*nx,weight)
!    call solver_smp(model,data,slowness_lop,cgstep,niter,Wop=weight_lop,mmov=mmov,rmov=rmov,verb=.true.)
     call solver_smp(model,data,slowness_lop,cgstep,niter,Wop=weight_lop,mmov=mmov,verb=.true.)
!     call solver_smp(model,data,slowness_lop,cgstep,niter,Wop=weight_lop,verb=.true.)
!     call solver_smp(model,data,slowness_rms_lop,cgstep,niter,Wop=weight_lop,mmov=mmov,verb=.true.)
  else
     call solver_smp(model,data,slowness_lop,cgstep,niter,mmov=mmov,rmov=rmov,verb=.true.)
  end if
!  call dot_test(slowness_rms_lop,nt*nm,nt*nh*nx,dot1,dot2)
  call slowness_close()

!  call to_history("n2",nx)
!  call to_history("d2",dx)
!  call to_history("o2",x0)
   
  call to_history("n2",nm)
  call to_history("d2",dm)
  call to_history("o2",m0)
   
  call to_history("n3",1)
  call to_history("d3",1)
  call to_history("o3",1)
  
  call to_history("n1",nt,"rmov");call to_history("n2",nh,"rmov");call to_history("n3",nx,"rmov");call to_history("n4",niter,"rmov")
  call to_history("d1",dt,"rmov");call to_history("d2",dh,"rmov");call to_history("d3",dx,"rmov");call to_history("d4",1,    "rmov")
  call to_history("o1",t0,"rmov");call to_history("o2",h0,"rmov");call to_history("o3",x0,"rmov");call to_history("n4",1,    "rmov")
  
  call to_history("n1",nt,"mmov");call to_history("n2",nm,"mmov");call to_history("n3",niter,"mmov")
  call to_history("d1",dt,"mmov");call to_history("d2",dm,"mmov");call to_history("d3",1,    "mmov")
  call to_history("o1",t0,"mmov");call to_history("o2",m0,"mmov");call to_history("n3",1,    "mmov")
 
!  call to_history("n1",nt,"mmov");call to_history("n2",nx,"mmov");call to_history("n3",niter,"mmov")
!  call to_history("d1",dt,"mmov");call to_history("d2",dx,"mmov");call to_history("d3",1,    "mmov")
!  call to_history("o1",t0,"mmov");call to_history("o2",x0,"mmov");call to_history("n3",1,    "mmov")
  
  call sep_write(model)
  call sep_write(rmov,"rmov")
  call sep_write(mmov,"mmov")

  deallocate (model,data,s0,rmov,mmov)
  
  call exit (0)
  
end program SLOWNESS
