program Slowness_reg_test

  use sep
  use dottest
  use cgstep_mod
  use slowness_1D_save3
  use weight_mod
  use igrad2axis
  use solver_reg_mod

  implicit none 
  integer                                 :: ndim, nt, nx, nh, nm, niter, stat
  real                                    :: t0,dt,x0,dx,h0,dh,dm,mf,m0,maxoffset,eps
  real,    dimension(2)                   :: dot1,dot2
  real,    dimension (:)    , allocatable :: model   ! Estimated velocity perturbations
  real,    dimension (:)    , allocatable :: data    ! CMP gathers
  real,    dimension (:)    , allocatable :: weight  ! Weighting function
  real,    dimension (:)    , pointer     :: s0      ! Initial 1D velocity
  integer                                 :: j, verb
  integer, dimension (:)    , allocatable :: n
  real,    dimension (:)    , allocatable :: d, o
  real,    dimension (:,:)  , allocatable :: mmov ! Movies

  call initpar  ()                   ! start processing parameters
  call sep_init (SOURCE)
  ndim=sep_dimension()
  allocate(n(ndim),o(ndim),d(ndim))
  
  call from_history('n',n)
  call from_history('d',d)
  call from_history('o',o)
  call from_param('verb',verb,0)
  call from_param('niter',niter,5)
  call from_param('eps',eps,1.)
  
  nt=n(1);t0=o(1);dt=d(1) ! Time
  nh=n(2);h0=o(2);dh=d(2) ! Offset
  nx=n(3);x0=o(3);dx=d(3) ! CMP
  
  maxoffset=h0+nh*dh       
  m0=x0-maxoffset/2       ! New origin of the model
  mf=x0+dx*nx+maxoffset/2 ! End of the model
  nm=ceiling((mf-m0)/dx)  ! Final size of the velocity model
  dm=dx                   ! Assumes CMP and velocity on the same grid
 
  if (verb.ge.1) write(0,*)
  if (verb.ge.1) write(0,*)
  if (verb.ge.1) write(0,*) '< ========================= >'
  if (verb.ge.1) write(0,*) '< = MODEL SPACE PARAMETERS= >'
  if (verb.ge.1) write(0,*) '< ========================= >'
  if (verb.ge.1) write(0,*) '< = o-time    = >',t0
  if (verb.ge.1) write(0,*) '< = d-time    = >',dt
  if (verb.ge.1) write(0,*) '< = n-time    = >',nt
  if (verb.ge.1) write(0,*) '< = o-cmp     = >',m0
  if (verb.ge.1) write(0,*) '< = n-cmp     = >',nm
  if (verb.ge.1) write(0,*) '< = d-cmp     = >',dm
  if (verb.ge.1) write(0,*) '< ========================= >'
  if (verb.ge.1) write(0,*)
  if (verb.ge.1) write(0,*)
  if (verb.ge.1) write(0,*) '< ========================= >'
  if (verb.ge.1) write(0,*) '< = DATA SPACE PARAMETERS = >'
  if (verb.ge.1) write(0,*) '< ========================= >'
  if (verb.ge.1) write(0,*) '< = o-time    = >',t0
  if (verb.ge.1) write(0,*) '< = d-time    = >',dt
  if (verb.ge.1) write(0,*) '< = n-time    = >',nt
  if (verb.ge.1) write(0,*) '< = o-offset  = >',h0
  if (verb.ge.1) write(0,*) '< = d-offset  = >',dh
  if (verb.ge.1) write(0,*) '< = n-offset  = >',nh
  if (verb.ge.1) write(0,*) '< = o-cmp     = >',x0
  if (verb.ge.1) write(0,*) '< = d-cmp     = >',dx
  if (verb.ge.1) write(0,*) '< = n-cmp     = >',nx
  if (verb.ge.1) write(0,*) '< ========================= >'
  if (verb.ge.1) write(0,*)
  if (verb.ge.1) write(0,*)
  if (verb.ge.1) write(0,*) '< ========================= >'
  if (verb.ge.1) write(0,*) '< = INVERSION  PARAMETERS = >'
  if (verb.ge.1) write(0,*) '< ========================= >'
  if (verb.ge.1) write(0,*) '< = niter    = >',niter
  if (verb.ge.1) write(0,*) '< = eps      = >',eps
  if (verb.ge.1) write(0,*) '< ========================= >'
  if (verb.ge.1) write(0,*)
  if (verb.ge.1) write(0,*)
  
  allocate (data(nt*nh*nx), model(nt*nm), s0(nt), mmov(nt*nm,niter))
     
  if (exist_file("weight")) then
     allocate(weight(nt*nh*nx))
     call sep_read(weight,'weight')
  end if

  call sep_read(data)
  call sep_read(s0,'s0')

  call slowness_init(t0,dt,nt,x0,dx,nx,m0,dm,nm,h0,dh,nh,s0,verb)
  call igrad2axis_init(nt,nm)
!  call dot_test(slowness_lop,nt*nm,nt*nh*nx,dot1,dot2)
  if (exist_file("weight")) then
     call weight_init(nt*nh*nx,weight)
     call solver_reg(model,data,slowness_lop,igrad2axis_lop,cgstep,nm*nt,niter,eps,Wop=weight_lop,mmov=mmov,verb=.true.)
  else
     call solver_reg(model,data,slowness_lop,igrad2axis_lop,cgstep,nm*nt,niter,eps,mmov=mmov,verb=.true.)
  end if
  call slowness_close()
   
  call to_history("n2",nm)
  call to_history("d2",dm)
  call to_history("o2",m0)
   
  call to_history("n3",1)
  call to_history("d3",1)
  call to_history("o3",1)
    
  call to_history("n1",nt,   "mmov")
  call to_history("n2",nm,   "mmov")
  call to_history("n3",niter,"mmov")
  call to_history("d1",dt,   "mmov")
  call to_history("d2",dm,   "mmov")
  call to_history("d3",1,    "mmov")
  call to_history("o1",t0,   "mmov")
  call to_history("o2",m0,   "mmov")
  call to_history("n3",1,    "mmov")
  
  call sep_write(model)
  call sep_write(mmov,"mmov")

  deallocate (model,data,s0,mmov)
  
  call exit (0)
  
end program SLOWNESS_REG_TEST
